using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Write1OutputValue
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Web reference to the system
        /// </summary>
        private MSXExxxxWeb.MSXExxxx sys;

        /// <summary>
        /// Contains the number of available outputs
        /// </summary>
        private int anaOutCount;

        /// <summary>
        /// List of available channels
        /// </summary>
        private List<String> availableChannels;

        /// <summary>
        /// Store the polarity of the channels
        /// </summary>
        private int[] channelPolarity;

        /// <summary>
        /// Currently selected index
        /// </summary>
        private int currentSelectedIndex;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
            this.sys = null;
            this.anaOutCount = -1;
            this.availableChannels = new List<string>();
            this.currentSelectedIndex = -1;
            this.channelPolarity = null;
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on ok button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void okButton_Click(object sender, EventArgs e)
        {
            ulong channelNumber = 0;
            this.sys = new MSXExxxxWeb.MSXExxxx();
            this.sys.Url = "http://" + this.ipTextBox.Text + ":5555";
            this.sys.Timeout = 2000;

            // default
            this.controlPanel.Enabled = false;

            try
            {
                MSXExxxxWeb.DefaultResponse response = this.sys.AnalogOutputGetNumberOfChannels(0, out channelNumber);
                if ((response.iReturnValue != 0) || (response.syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputGetNumberOfChannels. iReturnValue : " + response.iReturnValue.ToString() + " syserrno : " + response.syserrno.ToString());
                    return;
                }
                this.anaOutCount = (int)channelNumber;

                // Update graphics
                this.anaOutCountLabel.Text = this.anaOutCount.ToString();
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputGetNumberOfChannels exception : " + ex.Message);
                return;
            }

            // create the polarity
            this.channelPolarity = new int[this.anaOutCount];
            for (int i = 0; i < this.channelPolarity.Length; i++)
            {
                this.channelPolarity[i] = 0; // unipolar
            }

            // Create the database for the channel combo box
            this.availableChannels.Clear();
            for (int i = 0; i < (int)(this.anaOutCount); i++)
            {
                this.availableChannels.Add(i.ToString());
            }
            this.channelIndexComboBox.DataSource = this.availableChannels;

            // enable controls
            this.controlPanel.Enabled = true;

            // Set the trigger source selected index
            this.triggerSourceComboBox.SelectedIndex = 0;
            this.polarityComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// User changed the selected index
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void channelIndexComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            ulong channelValue = 0;

            // security check
            if (this.channelIndexComboBox.SelectedIndex < 0)
            {
                return;
            }

            if (this.sys == null)
            {
                return;
            }

            if (this.channelPolarity == null)
            {
                return;
            }

            // Set the selected index
            this.currentSelectedIndex = this.channelIndexComboBox.SelectedIndex;

            if (this.currentSelectedIndex >= this.channelPolarity.Length)
            {
                return;
            }

            // Get the current value of the channel
            try
            {
                MSXExxxxWeb.DefaultResponse response = null;
                response = this.sys.AnalogOutputGetChannelValue((ulong)this.currentSelectedIndex, 0, out channelValue);
                if ((response.iReturnValue != 0) || (response.syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputGetChannelValue. iReturnValue : " + response.iReturnValue.ToString() + " syserrno : " + response.syserrno.ToString());
                    return;
                }
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputGetChannelValue exception : " + ex.Message);
                return;
            }

            // Set the polarity
            this.polarityComboBox.SelectedIndex = this.channelPolarity[this.currentSelectedIndex];

            // update the value
            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                channelValue = channelValue - 32767;
            }
            
            this.valueTrackBar.Value = (int)channelValue;
            this.valueLabel.Text = ConstructValueLabelContent(channelValue);
        }

        /// <summary>
        /// Construct a string value
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private String ConstructValueLabelContent(ulong value)
        {
            double fvalue = 0;

            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                fvalue = (((double)value) * 10.0) / 32767.0;
            }
            else
            {
                fvalue = (((double)value - 32767.0) * 10.0) / 32767.0;
            }
            return value.ToString() + " (" + String.Format("{0:0.###}", fvalue) + " V)";
        }

        /// <summary>
        /// Update the graphics
        /// </summary>
        private void SetAsDisconnected()
        {
            this.sys = null;
            this.currentSelectedIndex = -1;
            this.anaOutCountLabel.Text = "-";
            this.controlPanel.Enabled = false;
            this.channelPolarity = null;
            this.availableChannels.Clear();
            this.channelIndexComboBox.DataSource = null;
            this.channelIndexComboBox.Update();
        }

        /// <summary>
        /// Used when scrolling
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void valueTrackBar_Scroll(object sender, EventArgs e)
        {
            ulong channelValue = (ulong)this.valueTrackBar.Value;
            int triggerMode = 0;

            switch (this.triggerSourceComboBox.SelectedIndex)
            {
                case 0:
                    triggerMode = 0;
                    break;
                case 1:
                    triggerMode = 4;
                    break;
                default:
                    triggerMode = 0;
                    break;
            }

            // security check
            if (this.currentSelectedIndex < 0)
            {
                return;
            }

            if (this.sys == null)
            {
                return;
            }

            if (triggerMode < 0)
            {
                return;
            }

            if (this.currentSelectedIndex >= this.channelPolarity.Length)
            {
                return;
            }

            // set the value
            try
            {
                int syserrno = 0;
                int retval = 0;
                retval = this.sys.AnalogOutputWrite1Value((ulong)this.currentSelectedIndex,
                    0, (ulong)this.channelPolarity[this.currentSelectedIndex], (ulong)triggerMode, 0, 0,
                    channelValue,
                    0, 0, 0, out syserrno);
                if ((retval != 0) || (syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputWrite1Value. retval : " + retval.ToString() + " syserrno : " + syserrno.ToString());
                    return;
                }
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputWrite1Value exception : " + ex.Message);
                return;
            }

            // update the value
            this.valueLabel.Text = ConstructValueLabelContent(channelValue);
        }

        /// <summary>
        /// Click on trigger outputs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void triggerOutputsButton_Click(object sender, EventArgs e)
        {
            // security check
            if (this.sys == null)
            {
                return;
            }

            // trigger the outputs
            try
            {
                int syserrno = 0;
                int retval = 0;
                retval = this.sys.AnalogOutputTriggerOutputs(0, out syserrno);
                if ((retval != 0) || (syserrno != 0))
                {
                    this.SetAsDisconnected();
                    MessageBox.Show("AnalogOutputTriggerOutputs. retval : " + retval.ToString() + " syserrno : " + syserrno.ToString());
                    return;
                }
            }
            catch (Exception ex)
            {
                this.SetAsDisconnected();
                MessageBox.Show("AnalogOutputTriggerOutputs exception : " + ex.Message);
                return;
            }
        }

        /// <summary>
        /// User change the polarity
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void polarityComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // security check
            if ( polarityComboBox.SelectedIndex < 0)
            {
                return;
            }

            if (this.currentSelectedIndex < 0)
            {
                return;
            }

            if ( this.currentSelectedIndex >= this.channelPolarity.Length )
            {
                return;
            }

            if (this.sys == null)
            {
                return;
            }

            if (this.channelPolarity == null)
            {
                return;
            }

            // Save the polarity
            this.channelPolarity[this.currentSelectedIndex] = this.polarityComboBox.SelectedIndex;

            // Change the range
            if (this.channelPolarity[this.currentSelectedIndex] == 0)
            {
                if (this.valueTrackBar.Value > 32767)
                {
                    this.valueTrackBar.Value = 32767;
                    this.valueLabel.Text = ConstructValueLabelContent(32767);
                }
                this.valueTrackBar.Maximum = 32767;
            }
            else
            {
                this.valueTrackBar.Maximum = 65535;
            }
        }
    }
}